@@ -7,11 +7,13 @@ import collections
 import enum
 import inspect
 import logging
+from typing import ClassVar
 
 from plinth import cfg
 from plinth.signals import post_app_loading
 
 from . import clients as clients_module
+from . import db
 
 logger = logging.getLogger(__name__)
 
@@ -39,14 +41,15 @@ class App:
 
     """
 
-    app_id = None
+    app_id: str | None = None
 
-    can_be_disabled = True
+    can_be_disabled: bool = True
 
-    locked = False  # Whether user interaction with the app is allowed.
+    locked: bool = False  # Whether user interaction with the app is allowed.
     # XXX: Lockdown the application UI by implementing a middleware
 
-    _all_apps = collections.OrderedDict()
+    _all_apps: ClassVar[collections.OrderedDict[
+        str, 'App']] = collections.OrderedDict()
 
     class SetupState(enum.Enum):
         """Various states of app being setup."""
@@ -133,6 +136,11 @@ class App:
         for component in self.components.values():
             component.setup(old_version=old_version)
 
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        for component in self.components.values():
+            component.uninstall()
+
     def get_setup_state(self) -> SetupState:
         """Return whether the app is not setup or needs upgrade."""
         current_version = self.get_setup_version()
@@ -150,8 +158,9 @@ class App:
         from . import models
 
         try:
-            app_entry = models.Module.objects.get(pk=self.app_id)
-            return app_entry.setup_version
+            with db.lock:
+                app_entry = models.Module.objects.get(pk=self.app_id)
+                return app_entry.setup_version
         except models.Module.DoesNotExist:
             return 0
 
@@ -166,8 +175,9 @@ class App:
         """Set the app's setup version."""
         from . import models
 
-        models.Module.objects.update_or_create(
-            pk=self.app_id, defaults={'setup_version': version})
+        with db.lock:
+            models.Module.objects.update_or_create(
+                pk=self.app_id, defaults={'setup_version': version})
 
     def enable(self):
         """Enable all the components of the app."""
@@ -201,10 +211,11 @@ class App:
     def diagnose(self):
         """Run diagnostics and return results.
 
-        Return value must be a list of results. Each result is a two-tuple with
-        first value as user visible description of the test followed by the
+        Return value must be a list of results. Each result is a
+        :class:`~plinth.modules.diagnostics.check.DiagnosticCheck` with a
+        unique check_id, a user visible description of the test, and the
         result. The test result is a string enumeration from 'failed',
-        'passed', 'error' and 'warning'.
+        'passed', 'error', 'warning' and 'not_done'.
 
         Results are typically collected by diagnosing each component of the app
         and then supplementing the results with any app level diagnostic tests.
@@ -280,6 +291,9 @@ class Component:
     def setup(self, old_version):
         """Run operations to install and configure the component."""
 
+    def uninstall(self):
+        """De-configure and uninstall the component."""
+
     def enable(self):
         """Run operations to enable the component."""
 
@@ -290,10 +304,11 @@ class Component:
     def diagnose():
         """Run diagnostics and return results.
 
-        Return value must be a list of results. Each result is a two-tuple with
-        first value as user visible description of the test followed by the
-        result. The test result is a string enumeration from 'failed', 'passed'
-        and 'error'.
+        Return value must be a list of results. Each result is a
+        :class:`~plinth.modules.diagnostics.check.DiagnosticCheck` with a
+        unique check_id, a user visible description of the test, and the
+        result. The test result is a string enumeration from 'failed',
+        'passed', 'error', 'warning' and 'not_done'.
 
         Also see :meth:`.has_diagnostics`.
 
