@@ -1,17 +1,13 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-Component for managing a background daemon or any systemd unit.
-"""
+"""Component for managing a background daemon or any systemd unit."""
 
 import socket
 import subprocess
 
 import psutil
-from django.utils.text import format_lazy
-from django.utils.translation import gettext as _
-from django.utils.translation import gettext_lazy
+from django.utils.translation import gettext_noop
 
-from plinth import action_utils, actions, app
+from plinth import action_utils, app
 
 
 class Daemon(app.LeaderComponent):
@@ -70,15 +66,17 @@ class Daemon(app.LeaderComponent):
 
     def enable(self):
         """Run operations to enable the daemon/unit."""
-        actions.superuser_run('service', ['enable', self.unit])
+        from plinth.privileged import service as service_privileged
+        service_privileged.enable(self.unit)
         if self.alias:
-            actions.superuser_run('service', ['enable', self.alias])
+            service_privileged.enable(self.alias)
 
     def disable(self):
         """Run operations to disable the daemon/unit."""
-        actions.superuser_run('service', ['disable', self.unit])
+        from plinth.privileged import service as service_privileged
+        service_privileged.disable(self.unit)
         if self.alias:
-            actions.superuser_run('service', ['disable', self.alias])
+            service_privileged.disable(self.alias)
 
     def is_running(self):
         """Return whether the daemon/unit is running."""
@@ -99,12 +97,15 @@ class Daemon(app.LeaderComponent):
 
     def _diagnose_unit_is_running(self):
         """Check if a daemon is running."""
-        result = 'passed' if self.is_running() else 'failed'
+        from plinth.modules.diagnostics.check import DiagnosticCheck, Result
 
-        template = gettext_lazy('Service {service_name} is running')
-        testname = format_lazy(template, service_name=self.unit)
+        check_id = f'daemon-running-{self.unit}'
+        result = Result.PASSED if self.is_running() else Result.FAILED
 
-        return [testname, result]
+        description = gettext_noop('Service {service_name} is running')
+        parameters = {'service_name': self.unit}
+
+        return DiagnosticCheck(check_id, description, result, parameters)
 
 
 class RelatedDaemon(app.FollowerComponent):
@@ -151,18 +152,23 @@ def diagnose_port_listening(port, kind='tcp', listen_address=None):
     information.
 
     """
+    from plinth.modules.diagnostics.check import DiagnosticCheck, Result
+
     result = _check_port(port, kind, listen_address)
 
+    parameters = {'kind': kind, 'port': port}
     if listen_address:
-        template = gettext_lazy(
+        parameters['listen_address'] = listen_address
+        check_id = f'daemon-listening-address-{kind}-{port}-{listen_address}'
+        description = gettext_noop(
             'Listening on {kind} port {listen_address}:{port}')
-        testname = format_lazy(template, kind=kind,
-                               listen_address=listen_address, port=port)
     else:
-        template = gettext_lazy('Listening on {kind} port {port}')
-        testname = format_lazy(template, kind=kind, port=port)
+        check_id = f'daemon-listening-{kind}-{port}'
+        description = gettext_noop('Listening on {kind} port {port}')
 
-    return [testname, 'passed' if result else 'failed']
+    return DiagnosticCheck(check_id, description,
+                           Result.PASSED if result else Result.FAILED,
+                           parameters)
 
 
 def _check_port(port, kind='tcp', listen_address=None):
@@ -211,6 +217,8 @@ def _check_port(port, kind='tcp', listen_address=None):
 
 def diagnose_netcat(host, port, input='', negate=False):
     """Run a diagnostic using netcat."""
+    from plinth.modules.diagnostics.check import DiagnosticCheck, Result
+
     try:
         process = subprocess.Popen(['nc', host, str(port)],
                                    stdin=subprocess.PIPE,
@@ -218,17 +226,18 @@ def diagnose_netcat(host, port, input='', negate=False):
                                    stderr=subprocess.PIPE)
         process.communicate(input=input.encode())
         if process.returncode != 0:
-            result = 'failed'
+            result = Result.FAILED if not negate else Result.PASSED
         else:
-            result = 'passed'
-
-        if negate:
-            result = 'failed' if result == 'passed' else 'passed'
+            result = Result.PASSED if not negate else Result.FAILED
     except Exception:
-        result = 'failed'
+        result = Result.FAILED
 
-    test = _('Connect to {host}:{port}')
+    check_id = f'daemon-netcat-{host}-{port}'
+    description = gettext_noop('Connect to {host}:{port}')
+    parameters = {'host': host, 'port': port, 'negate': negate}
     if negate:
-        test = _('Cannot connect to {host}:{port}')
+        check_id = f'daemon-netcat-negate-{host}-{port}'
+        description = gettext_noop('Cannot connect to {host}:{port}')
 
-    return [test.format(host=host, port=port), result]
+    return DiagnosticCheck(check_id, description.format(host=host, port=port),
+                           result, parameters)
