@@ -12,6 +12,7 @@ from plinth import app
 from plinth.modules.apache.components import (Uwsgi, Webserver, check_url,
                                               diagnose_url,
                                               diagnose_url_on_all)
+from plinth.modules.diagnostics.check import DiagnosticCheck, Result
 
 
 def test_webserver_init():
@@ -72,18 +73,22 @@ def test_webserver_diagnose(diagnose_url_on_all, diagnose_url):
     """Test running diagnostics."""
 
     def on_all_side_effect(url, check_certificate, expect_redirects):
-        return [('test-result-' + url, 'success')]
+        return [
+            DiagnosticCheck('test-all-id', 'test-result-' + url, 'success')
+        ]
 
     def side_effect(url, check_certificate):
-        return ('test-result-' + url, 'success')
+        return DiagnosticCheck('test-id', 'test-result-' + url, 'success')
 
     diagnose_url_on_all.side_effect = on_all_side_effect
     diagnose_url.side_effect = side_effect
     webserver1 = Webserver('test-webserver', 'test-config',
                            urls=['{host}url1', 'url2'], expect_redirects=True)
     results = webserver1.diagnose()
-    assert results == [('test-result-{host}url1', 'success'),
-                       ('test-result-url2', 'success')]
+    assert results == [
+        DiagnosticCheck('test-all-id', 'test-result-{host}url1', 'success'),
+        DiagnosticCheck('test-id', 'test-result-url2', 'success')
+    ]
     diagnose_url_on_all.assert_has_calls(
         [call('{host}url1', check_certificate=False, expect_redirects=True)])
     diagnose_url.assert_has_calls([call('url2', check_certificate=False)])
@@ -241,17 +246,22 @@ def test_diagnose_url(get_addresses, check):
         'wrapper': 'test-wrapper',
         'expected_output': 'test-expected'
     }
-    check.return_value = 'passed'
+    parameters = {key: args[key] for key in ['url', 'kind']}
+    check.return_value = True
     result = diagnose_url(**args)
-    assert result == ['Access URL https://localhost/test on tcp4', 'passed']
+    assert result == DiagnosticCheck(
+        'apache-url-kind-https://localhost/test-4',
+        'Access URL {url} on tcp{kind}', Result.PASSED, parameters)
 
-    check.return_value = 'failed'
+    check.return_value = False
     result = diagnose_url(**args)
-    assert result == ['Access URL https://localhost/test on tcp4', 'failed']
+    assert result == DiagnosticCheck(
+        'apache-url-kind-https://localhost/test-4',
+        'Access URL {url} on tcp{kind}', Result.FAILED, parameters)
 
     del args['kind']
     args['url'] = 'https://{host}/test'
-    check.return_value = 'passed'
+    check.return_value = True
     get_addresses.return_value = [{
         'kind': '4',
         'address': 'test-host-1',
@@ -263,10 +273,24 @@ def test_diagnose_url(get_addresses, check):
         'numeric': False,
         'url_address': 'test-host-2'
     }]
-    result = diagnose_url_on_all(**args)
-    assert result == [
-        ['Access URL https://test-host-1/test on tcp4', 'passed'],
-        ['Access URL https://test-host-2/test on tcp6', 'passed'],
+    parameters = [
+        {
+            'url': 'https://test-host-1/test',
+            'kind': '4'
+        },
+        {
+            'url': 'https://test-host-2/test',
+            'kind': '6'
+        },
+    ]
+    results = diagnose_url_on_all(**args)
+    assert results == [
+        DiagnosticCheck('apache-url-kind-https://test-host-1/test-4',
+                        'Access URL {url} on tcp{kind}', Result.PASSED,
+                        parameters[0]),
+        DiagnosticCheck('apache-url-kind-https://test-host-2/test-6',
+                        'Access URL {url} on tcp{kind}', Result.PASSED,
+                        parameters[1]),
     ]
 
 
@@ -278,7 +302,7 @@ def test_check_url(run):
     extra_args = {'env': None, 'check': True, 'stdout': -1, 'stderr': -1}
 
     # Basic
-    assert check_url(url) == 'passed'
+    assert check_url(url)
     run.assert_called_with(basic_command + [url], **extra_args)
 
     # Wrapper
@@ -311,15 +335,16 @@ def test_check_url(run):
     exception = subprocess.CalledProcessError(returncode=1, cmd=['curl'])
     run.side_effect = exception
     run.side_effect.stdout = b'500'
-    assert check_url(url) == 'failed'
+    assert not check_url(url)
 
     # Return code 401, 405
     run.side_effect = exception
     run.side_effect.stdout = b' 401 '
-    assert check_url(url) == 'passed'
+    assert check_url(url)
     run.side_effect.stdout = b'405\n'
-    assert check_url(url) == 'passed'
+    assert check_url(url)
 
     # Error
     run.side_effect = FileNotFoundError()
-    assert check_url(url) == 'error'
+    with pytest.raises(FileNotFoundError):
+        assert check_url(url)
