@@ -6,11 +6,13 @@ FreedomBox app to configure system date and time.
 import subprocess
 
 from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext_noop
 
 from plinth import app as app_module
 from plinth import menu
-from plinth.daemon import Daemon
+from plinth.daemon import Daemon, RelatedDaemon
 from plinth.modules.backups.components import BackupRestore
+from plinth.modules.diagnostics.check import DiagnosticCheck, Result
 from plinth.package import Packages
 
 from . import manifest
@@ -75,6 +77,10 @@ class DateTimeApp(app_module.App):
         packages = Packages('packages-datetime', ['systemd-timesyncd'])
         self.add(packages)
 
+        daemon = RelatedDaemon('daemon-datetime-timedated',
+                               'systemd-timedated')
+        self.add(daemon)
+
         if self._is_time_managed():
             daemon = Daemon('daemon-datetime', 'systemd-timesyncd')
             self.add(daemon)
@@ -103,13 +109,15 @@ class DateTimeApp(app_module.App):
 
 def _diagnose_time_synchronized():
     """Check whether time is synchronized to NTP server."""
-    result = 'failed'
+    result = Result.FAILED
     try:
         output = subprocess.check_output(
             ['timedatectl', 'show', '--property=NTPSynchronized', '--value'])
         if 'yes' in output.decode():
-            result = 'passed'
+            result = Result.PASSED
     except subprocess.CalledProcessError:
         pass
 
-    return [_('Time synchronized to NTP server'), result]
+    return DiagnosticCheck('datetime-ntp-sync',
+                           gettext_noop('Time synchronized to NTP server'),
+                           result)
