@@ -3,7 +3,6 @@
 
 import json
 import logging
-from typing import Tuple
 
 from django.urls import reverse_lazy
 from django.utils.translation import gettext_lazy as _
@@ -51,7 +50,7 @@ class EjabberdApp(app_module.App):
 
     app_id = 'ejabberd'
 
-    _version = 7
+    _version = 8
 
     def __init__(self):
         """Create components for the app."""
@@ -149,9 +148,14 @@ class EjabberdApp(app_module.App):
         if not old_version:
             self.enable()
 
-        # Configure STUN/TURN only if there's a valid TLS domain set for Coturn
-        configuration = self.get_component('turn-ejabberd').get_configuration()
-        update_turn_configuration(configuration, force=True)
+        if not old_version or get_turn_configuration()[1]:
+            # Configure STUN/TURN only if there's a valid TLS domain set for
+            # Coturn. Do this if app is being freshly installed or if it is
+            # previously installed and configured to use STUN/TURN
+            # auto-management.
+            configuration = self.get_component(
+                'turn-ejabberd').get_configuration()
+            update_turn_configuration(configuration, force=True)
 
 
 class EjabberdTurnConsumer(TurnConsumer):
@@ -226,7 +230,7 @@ def update_turn_configuration(config: TurnConfiguration, managed=True,
     privileged.configure_turn(json.loads(config.to_json()), managed)
 
 
-def get_turn_configuration() -> Tuple[TurnConfiguration, bool]:
+def get_turn_configuration() -> tuple[TurnConfiguration, bool]:
     """Get the latest STUN/TURN configuration."""
     tc, managed = privileged.get_turn_config()
     return TurnConfiguration(tc['domain'], tc['uris'],
