@@ -6,6 +6,7 @@ import logging
 import pathlib
 
 from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext_noop
 
 from plinth import app as app_module
 from plinth import cfg, menu
@@ -13,6 +14,7 @@ from plinth.config import DropinConfigs
 from plinth.modules import names
 from plinth.modules.apache.components import diagnose_url
 from plinth.modules.backups.components import BackupRestore
+from plinth.modules.diagnostics.check import DiagnosticCheck, Result
 from plinth.modules.names.components import DomainType
 from plinth.package import Packages
 from plinth.signals import domain_added, domain_removed, post_app_loading
@@ -97,7 +99,10 @@ class LetsEncryptApp(app_module.App):
 
         if not results:
             results.append(
-                (_('Cannot test: No domains are configured.'), 'warning'))
+                DiagnosticCheck(
+                    'letsencrypt-cannot-test',
+                    gettext_noop('Cannot test: No domains are configured.'),
+                    Result.WARNING))
 
         return results
 
