@@ -1,23 +1,21 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to configure MediaWiki.
-"""
+"""FreedomBox app to configure MediaWiki."""
 
-import re
+import pathlib
 from urllib.parse import urlparse
 
 from django.utils.translation import gettext_lazy as _
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import frontpage, menu
+from plinth.config import DropinConfigs
 from plinth.daemon import Daemon
 from plinth.modules.apache.components import Webserver
 from plinth.modules.backups.components import BackupRestore
 from plinth.modules.firewall.components import Firewall
 from plinth.package import Packages
 
-from . import manifest
+from . import manifest, privileged
 
 _description = [
     _('MediaWiki is the wiki engine that powers Wikipedia and other WikiMedia '
@@ -34,7 +32,6 @@ _description = [
       'logged in can make changes to the content.')
 ]
 
-STATIC_CONFIG_FILE = '/etc/mediawiki/FreedomBoxStaticSettings.php'
 USER_CONFIG_FILE = '/etc/mediawiki/FreedomBoxSettings.php'
 
 
@@ -43,7 +40,7 @@ class MediaWikiApp(app_module.App):
 
     app_id = 'mediawiki'
 
-    _version = 10
+    _version = 12
 
     def __init__(self):
         """Create components for the app."""
@@ -73,6 +70,12 @@ class MediaWikiApp(app_module.App):
                             ['mediawiki', 'imagemagick', 'php-sqlite3'])
         self.add(packages)
 
+        dropin_configs = DropinConfigs('dropin-configs-mediawiki', [
+            '/etc/apache2/conf-available/mediawiki-freedombox.conf',
+            '/etc/mediawiki/FreedomBoxStaticSettings.php',
+        ])
+        self.add(dropin_configs)
+
         firewall = Firewall('firewall-mediawiki', info.name,
                             ports=['http', 'https'], is_external=True)
         self.add(firewall)
@@ -96,9 +99,15 @@ class MediaWikiApp(app_module.App):
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('mediawiki', ['setup'])
-        actions.superuser_run('mediawiki', ['update'])
-        self.enable()
+        privileged.setup()
+        privileged.update()
+        if not old_version:
+            self.enable()
+
+    def uninstall(self):
+        """De-configure and uninstall the app."""
+        super().uninstall()
+        privileged.uninstall()
 
 
 class Shortcut(frontpage.Shortcut):
@@ -107,20 +116,7 @@ class Shortcut(frontpage.Shortcut):
     def enable(self):
         """When enabled, check if MediaWiki is in private mode."""
         super().enable()
-        self.login_required = is_private_mode_enabled()
-
-
-def is_public_registration_enabled():
-    """Return whether public registration is enabled."""
-    output = actions.superuser_run('mediawiki',
-                                   ['public-registrations', 'status'])
-    return output.strip() == 'enabled'
-
-
-def is_private_mode_enabled():
-    """Return whether private mode is enabled or disabled."""
-    output = actions.superuser_run('mediawiki', ['private-mode', 'status'])
-    return output.strip() == 'enabled'
+        self.login_required = get_config()['enable_private_mode']
 
 
 def _get_config_value_in_file(setting_name, config_file):
@@ -128,31 +124,45 @@ def _get_config_value_in_file(setting_name, config_file):
     with open(config_file, 'r', encoding='utf-8') as config:
         for line in config:
             if line.startswith(setting_name):
-                return re.findall(r'["\'][^"\']*["\']', line)[0].strip('"\'')
+                return line.partition('=')[2].strip('\n ;\'"')
 
         return None
 
 
+def _get_static_config_file():
+    """Return the path for the file containing static settings."""
+    base_path = ('/usr/share/freedombox/etc/'
+                 'mediawiki/FreedomBoxStaticSettings.php')
+    for path in [
+            pathlib.Path(base_path),
+            pathlib.Path(__file__).parent / 'data' / base_path.lstrip('/')
+    ]:
+        if path.exists():
+            return path
+
+    raise RuntimeError('Unable to find static config file')
+
+
 def _get_config_value(setting_name):
     """Return a configuration value from multiple configuration files."""
     return _get_config_value_in_file(setting_name, USER_CONFIG_FILE) or \
-        _get_config_value_in_file(setting_name, STATIC_CONFIG_FILE)
+        _get_config_value_in_file(setting_name, _get_static_config_file())
 
 
-def get_default_skin():
-    """Return the value of the default skin."""
-    return _get_config_value('$wgDefaultSkin')
-
-
-def set_default_skin(skin):
-    """Set the value of the default skin."""
-    actions.superuser_run('mediawiki', ['set-default-skin', skin])
-
-
-def get_server_url():
-    """Return the value of the server URL."""
+def get_config():
+    """Return all the configuration settings."""
     server_url = _get_config_value('$wgServer')
-    return urlparse(server_url).netloc
+    create_permission = _get_config_value(
+        "$wgGroupPermissions['*']['createaccount']")
+    read_permission = _get_config_value("$wgGroupPermissions['*']['read']")
+    return {
+        'default_skin': _get_config_value('$wgDefaultSkin'),
+        'domain': urlparse(server_url).netloc,
+        'site_name': _get_config_value('$wgSitename') or 'Wiki',
+        'default_lang': _get_config_value('$wgLanguageCode') or None,
+        'enable_public_registrations': 'true' in create_permission,
+        'enable_private_mode': 'false' in read_permission,
+    }
 
 
 def set_server_url(domain):
@@ -161,15 +171,4 @@ def set_server_url(domain):
     if domain.endswith('.onion'):
         protocol = 'http'
 
-    actions.superuser_run('mediawiki',
-                          ['set-server-url', f'{protocol}://{domain}'])
-
-
-def get_site_name():
-    """Return the value of MediaWiki's site name."""
-    return _get_config_value('$wgSitename') or 'Wiki'
-
-
-def set_site_name(site_name):
-    """Set the value of $wgSitename."""
-    actions.superuser_run('mediawiki', ['set-site-name', site_name])
+    privileged.set_server_url(f'{protocol}://{domain}')
