@@ -1,25 +1,21 @@
 # SPDX-License-Identifier: AGPL-3.0-or-later
-"""
-FreedomBox app to manage storage.
-"""
+"""FreedomBox app to manage storage."""
 
 import base64
 import logging
-import subprocess
 
 import psutil
 from django.utils.translation import gettext_lazy as _
 from django.utils.translation import gettext_noop
 
-from plinth import actions
 from plinth import app as app_module
 from plinth import cfg, glib, menu
-from plinth.errors import ActionError, PlinthError
+from plinth.errors import PlinthError
 from plinth.modules.backups.components import BackupRestore
 from plinth.package import Packages
 from plinth.utils import format_lazy
 
-from . import manifest, udisks2
+from . import manifest, privileged, udisks2
 
 _description = [
     format_lazy(
@@ -56,8 +52,7 @@ class StorageApp(app_module.App):
         self.add(menu_item)
 
         packages = Packages('packages-storage',
-                            ['parted', 'udisks2', 'gir1.2-udisks-2.0'],
-                            skip_recommends=True)
+                            ['parted', 'udisks2', 'gir1.2-udisks-2.0'])
         self.add(packages)
 
         backup_restore = BackupRestore('backup-restore-storage',
@@ -67,29 +62,31 @@ class StorageApp(app_module.App):
     @staticmethod
     def post_init():
         """Perform post initialization operations."""
-        # Check every hour for low disk space, every 3 minutes in debug mode
-        interval = 180 if cfg.develop else 3600
-        glib.schedule(interval, warn_about_low_disk_space)
+        # Check every hour for low disk space
+        glib.schedule(3600, warn_about_low_disk_space)
 
         # Schedule initialization of UDisks2 initialization
         glib.schedule(3, udisks2.init, repeat=False)
 
+        # Check periodically for a read-only root filesystem
+        glib.schedule(600, _warn_about_read_only_filesystem)
+
     def setup(self, old_version):
         """Install and configure the app."""
         super().setup(old_version)
-        actions.superuser_run('storage', ['setup'])
+        privileged.setup()
         self.enable()
         disks = get_disks()
         root_device = get_root_device(disks)
         if is_expandable(root_device):
             try:
-                expand_partition(root_device)
-            except ActionError:
+                privileged.expand_partition(root_device)
+            except Exception:
                 pass
 
 
 def get_disks():
-    """Returns list of disks and their free space.
+    """Return list of disks and their free space.
 
     The primary source of information is UDisks' list of block devices.
     Information from df is used for free space available.
@@ -136,8 +133,8 @@ def get_mounts():
 def _get_disks_from_df():
     """Return the list of disks and free space available using 'df'."""
     try:
-        output = actions.superuser_run('storage', ['usage-info'])
-    except subprocess.CalledProcessError as exception:
+        output = privileged.usage_info()
+    except Exception as exception:
         logger.exception('Error getting disk information: %s', exception)
         return []
 
@@ -162,8 +159,8 @@ def _get_disks_from_df():
 
 
 def get_filesystem_type(mount_point='/'):
-    """Returns the type of the filesystem mounted at mountpoint."""
-    for partition in psutil.disk_partitions():
+    """Return the type of the filesystem mounted at mountpoint."""
+    for partition in psutil.disk_partitions(all=True):
         if partition.mountpoint == mount_point:
             return partition.fstype
 
@@ -204,19 +201,10 @@ def is_expandable(device):
         return False
 
     try:
-        output = actions.superuser_run('storage',
-                                       ['is-partition-expandable', device],
-                                       log_error=False)
-    except actions.ActionError:
+        return privileged.is_partition_expandable(device, _log_error=False)
+    except Exception:
         return False
 
-    return int(output.strip())
-
-
-def expand_partition(device):
-    """Expand a partition."""
-    actions.superuser_run('storage', ['expand-partition', device])
-
 
 def format_bytes(size):
     """Return human readable disk size from bytes."""
@@ -286,7 +274,7 @@ def get_error_message(error):
     return message_map.get(short_error, error)
 
 
-def warn_about_low_disk_space(request):
+def warn_about_low_disk_space(_data):
     """Warn about insufficient space on root partition."""
     from plinth.notification import Notification
 
@@ -356,3 +344,55 @@ def report_failing_drive(id, is_failing):
                                              'type': 'dismiss'
                                          }], data=data, group='admin')
     note.dismiss(should_dismiss=not is_failing)
+
+
+def is_partition_read_only(mount_point='/'):
+    """Return True if the partition is mounted read-only."""
+    for partition in psutil.disk_partitions(all=True):
+        if partition.mountpoint == mount_point:
+            return 'ro' in partition.opts.split(',')
+
+    raise ValueError('No such mount point')
+
+
+def _warn_about_read_only_filesystem(_data):
+    """Create a notification if the root filesystem is mounted read-only.
+
+    Remove the notification (if any) if the root filesystem is writable.
+    """
+    from plinth.notification import Notification
+
+    show = is_partition_read_only()
+    notification_id = 'storage-read-only-root-filesystem'
+
+    if not show:
+        try:
+            Notification.get(notification_id).delete()
+        except KeyError:
+            pass
+
+        return
+
+    message = gettext_noop(
+        # xgettext:no-python-format
+        'You cannot save configuration changes. Try rebooting the system. If '
+        'the problem persists after a reboot, check the storage device for '
+        'errors.')
+    title = gettext_noop('Read-only root filesystem')
+    data = {
+        'app_icon': 'fa-hdd-o',
+        'app_name': 'translate:' + gettext_noop('Storage'),
+    }
+
+    # This is a serious issue so the notification can't be dismissed.
+    actions = [{
+        'type': 'link',
+        'class': 'primary',
+        'text': gettext_noop('Go to Power'),
+        'url': 'power:index'
+    }]
+
+    Notification.update_or_create(id=notification_id, app_id='storage',
+                                  severity='error', title=title,
+                                  message=message, actions=actions, data=data,
+                                  group='admin')
