@@ -5,11 +5,10 @@ import enum
 import logging
 import pathlib
 import time
-from typing import Optional, Union
 
 import apt.cache
 from django.utils.translation import gettext as _
-from django.utils.translation import gettext_lazy
+from django.utils.translation import gettext_lazy, gettext_noop
 
 import plinth.privileged.packages as privileged
 from plinth import app as app_module
@@ -42,11 +41,11 @@ class Package(PackageExpression):
             self,
             name,
             optional: bool = False,
-            version: Optional[str] = None,  # ">=1.0,<2.0"
-            distribution: Optional[str] = None,  # Debian, Ubuntu
-            suite: Optional[str] = None,  # stable, testing
-            codename: Optional[str] = None,  # bullseye-backports
-            architecture: Optional[str] = None):  # arm64
+            version: str | None = None,  # ">=1.0,<2.0"
+            distribution: str | None = None,  # Debian, Ubuntu
+            suite: str | None = None,  # stable, testing
+            codename: str | None = None,  # bullseye-backports
+            architecture: str | None = None):  # arm64
         self.name = name
         self.optional = optional
         self.version = version
@@ -108,10 +107,10 @@ class Packages(app_module.FollowerComponent):
         REMOVE = 'remove'  # Remove the packages before installing the app
 
     def __init__(self, component_id: str,
-                 packages: list[Union[str, PackageExpression]],
+                 packages: list[str | PackageExpression],
                  skip_recommends: bool = False,
-                 conflicts: Optional[list[str]] = None,
-                 conflicts_action: Optional[ConflictsAction] = None):
+                 conflicts: list[str] | None = None,
+                 conflicts_action: ConflictsAction | None = None):
         """Initialize a new packages component.
 
         'component_id' should be a unique ID across all components of an app
@@ -203,41 +202,51 @@ class Packages(app_module.FollowerComponent):
 
     def diagnose(self):
         """Run diagnostics and return results."""
+        from plinth.modules.diagnostics.check import DiagnosticCheck, Result
+
         results = super().diagnose()
         cache = apt.Cache()
         for package_expression in self.package_expressions:
             try:
                 package_name = package_expression.actual()
             except MissingPackageError:
-                message = _('Package {expression} is not available for '
-                            'install').format(expression=package_expression)
-                results.append([message, 'failed'])
+                check_id = f'package-available-{package_expression}'
+                description = gettext_noop('Package {package_expression} is '
+                                           'not available for install')
+                parameters = {'package_expression': str(package_expression)}
+                results.append(
+                    DiagnosticCheck(check_id, description, Result.FAILED,
+                                    parameters))
                 continue
 
-            result = 'warning'
+            result = Result.WARNING
             latest_version = '?'
             if package_name in cache:
                 package = cache[package_name]
                 latest_version = package.candidate.version
                 if package.candidate.is_installed:
-                    result = 'passed'
+                    result = Result.PASSED
 
-            message = _('Package {package_name} is the latest version '
-                        '({latest_version})').format(
-                            package_name=package_name,
-                            latest_version=latest_version)
-            results.append([message, result])
+            check_id = f'package-latest-{package_name}'
+            description = gettext_noop('Package {package_name} is the latest '
+                                       'version ({latest_version})')
+            parameters = {
+                'package_name': package_name,
+                'latest_version': latest_version,
+            }
+            results.append(
+                DiagnosticCheck(check_id, description, result, parameters))
 
         return results
 
-    def find_conflicts(self) -> Optional[list[str]]:
+    def find_conflicts(self) -> list[str] | None:
         """Return list of conflicting packages installed on the system."""
         if not self.conflicts:
             return None
 
         return packages_installed(self.conflicts)
 
-    def has_unavailable_packages(self) -> Optional[bool]:
+    def has_unavailable_packages(self) -> bool | None:
         """Return whether any of the packages are not available.
 
         Returns True if one or more of the packages is not available in the
@@ -465,7 +474,7 @@ def filter_conffile_prompt_packages(packages):
     return privileged.filter_conffile_packages(list(packages))
 
 
-def packages_installed(candidates: Union[list, tuple]) -> list:
+def packages_installed(candidates: list | tuple) -> list:
     """Check which candidates are installed on the system.
 
     :param candidates: A list of package names.
