@@ -22,8 +22,17 @@ from selenium.webdriver.support.ui import WebDriverWait
 config = configparser.ConfigParser()
 config.read(pathlib.Path(__file__).with_name('config.ini'))
 
-config['DEFAULT']['url'] = os.environ.get('FREEDOMBOX_URL',
-                                          config['DEFAULT']['url']).rstrip('/')
+# Configuration to allow each pytest-xdist worker to hit a dedicated
+# app server. See .ci/functional-tests.yml for usage.
+worker = os.environ.get('PYTEST_XDIST_WORKER', 'master')
+if worker == 'master':
+    config['DEFAULT']['url'] = os.environ.get(
+        'FREEDOMBOX_URL', config['DEFAULT']['url']).rstrip('/')
+else:
+    # worker_ids are like gw0, gw1, ...
+    worker_number = int(worker.lstrip('gw')) + 1
+    config['DEFAULT']['url'] = os.environ[f'APP_SERVER_URL_{worker_number}']
+
 config['DEFAULT']['ssh_port'] = os.environ.get('FREEDOMBOX_SSH_PORT',
                                                config['DEFAULT']['ssh_port'])
 config['DEFAULT']['samba_port'] = os.environ.get(
@@ -277,7 +286,7 @@ def login_with_account(browser, url, username, password=None):
         if user_menu.text == username:
             return
 
-        visit(browser, '/plinth/accounts/logout/')
+        logout(browser)
 
     login_button = browser.links.find_by_href('/plinth/accounts/login/')
     if login_button:
@@ -306,7 +315,14 @@ def login_with_account(browser, url, username, password=None):
 
 def logout(browser):
     """Log out of the FreedomBox interface."""
-    visit(browser, '/plinth/accounts/logout/')
+    # Navigate to the home page if logout form is not found
+    if not browser.find_by_css('.form-logout'):
+        visit(browser, '/plinth/')
+
+    # We are not logged in if the home page does not contain logout form
+    if browser.find_by_css('.form-logout'):
+        browser.find_by_id('id_user_menu').click()
+        submit(browser, form_class='form-logout')
 
 
 #################
@@ -559,7 +575,7 @@ def networks_set_firewall_zone(browser, zone):
     edit_url = "/plinth/sys/networks/{}/edit/".format(network_id)
     browser.links.find_by_href(edit_url).first.click()
     browser.select('zone', zone)
-    browser.find_by_tag("form").first.find_by_tag('input')[-1].click()
+    submit(browser, form_class='form-connection-edit')
 
 
 ##################
