@@ -4,13 +4,23 @@ Test module for component managing system daemons and other systemd units.
 """
 
 import socket
+import subprocess
 from unittest.mock import Mock, call, patch
 
 import pytest
 
-from plinth.app import App, FollowerComponent
+from plinth.app import App, FollowerComponent, Info
 from plinth.daemon import (Daemon, RelatedDaemon, app_is_running,
                            diagnose_netcat, diagnose_port_listening)
+from plinth.modules.diagnostics.check import DiagnosticCheck, Result
+
+privileged_modules_to_mock = ['plinth.privileged.service']
+
+
+class AppTest(App):
+    """Test application that contains a daemon."""
+
+    app_id = 'test-app'
 
 
 @pytest.fixture(name='daemon')
@@ -19,6 +29,17 @@ def fixture_daemon():
     return Daemon('test-daemon', 'test-unit')
 
 
+@pytest.fixture(name='app_list')
+def fixture_app_list(daemon):
+    """A list of apps on which tests are to be run."""
+    app1 = AppTest()
+    app1.add(Info('test-app', 1))
+    app1.add(daemon)
+    with patch('plinth.app.App.list') as app_list:
+        app_list.return_value = [app1]
+        yield app_list
+
+
 def test_initialization():
     """Test that component is initialized properly."""
     with pytest.raises(ValueError):
@@ -56,25 +77,51 @@ def test_is_enabled(service_is_enabled, daemon):
     service_is_enabled.assert_has_calls([call('test-unit', strict_check=True)])
 
 
-@patch('plinth.actions.superuser_run')
-def test_enable(superuser_run, daemon):
+@patch('subprocess.run')
+@patch('subprocess.call')
+def test_enable(subprocess_call, subprocess_run, app_list, mock_privileged,
+                daemon):
     """Test that enabling the daemon works."""
     daemon.enable()
-    superuser_run.assert_has_calls([call('service', ['enable', 'test-unit'])])
+    subprocess_call.assert_has_calls(
+        [call(['systemctl', 'enable', 'test-unit'])])
+    subprocess_run.assert_any_call(['systemctl', 'start', 'test-unit'],
+                                   stdout=subprocess.DEVNULL, check=False)
 
+    subprocess_call.reset_mock()
     daemon.alias = 'test-unit-2'
     daemon.enable()
-    superuser_run.assert_has_calls([
-        call('service', ['enable', 'test-unit']),
-        call('service', ['enable', 'test-unit-2'])
+    subprocess_call.assert_has_calls([
+        call(['systemctl', 'enable', 'test-unit']),
+        call(['systemctl', 'enable', 'test-unit-2'])
     ])
+    subprocess_run.assert_any_call(['systemctl', 'start', 'test-unit'],
+                                   stdout=subprocess.DEVNULL, check=False)
+    subprocess_run.assert_any_call(['systemctl', 'start', 'test-unit-2'],
+                                   stdout=subprocess.DEVNULL, check=False)
 
 
-@patch('plinth.actions.superuser_run')
-def test_disable(superuser_run, daemon):
+@patch('subprocess.run')
+@patch('subprocess.call')
+def test_disable(subprocess_call, subprocess_run, mock_privileged, daemon):
     """Test that disabling the daemon works."""
     daemon.disable()
-    superuser_run.assert_has_calls([call('service', ['disable', 'test-unit'])])
+    subprocess_call.assert_has_calls(
+        [call(['systemctl', 'disable', 'test-unit'])])
+    subprocess_run.assert_any_call(['systemctl', 'stop', 'test-unit'],
+                                   stdout=subprocess.DEVNULL, check=False)
+
+    subprocess_call.reset_mock()
+    daemon.alias = 'test-unit-2'
+    daemon.disable()
+    subprocess_call.assert_has_calls([
+        call(['systemctl', 'disable', 'test-unit']),
+        call(['systemctl', 'disable', 'test-unit-2'])
+    ])
+    subprocess_run.assert_any_call(['systemctl', 'stop', 'test-unit'],
+                                   stdout=subprocess.DEVNULL, check=False)
+    subprocess_run.assert_any_call(['systemctl', 'stop', 'test-unit-2'],
+                                   stdout=subprocess.DEVNULL, check=False)
 
 
 @patch('plinth.action_utils.service_is_running')
@@ -94,22 +141,29 @@ def test_diagnose(port_listening, service_is_running, daemon):
     """Test running diagnostics."""
 
     def side_effect(port, kind):
-        return [f'test-result-{port}-{kind}', 'passed']
+        name = f'test-result-{port}-{kind}'
+        return DiagnosticCheck(name, name, Result.PASSED)
 
     daemon = Daemon('test-daemon', 'test-unit', listen_ports=[(8273, 'tcp4'),
                                                               (345, 'udp')])
     port_listening.side_effect = side_effect
     service_is_running.return_value = True
     results = daemon.diagnose()
-    assert results == [['Service test-unit is running', 'passed'],
-                       ['test-result-8273-tcp4', 'passed'],
-                       ['test-result-345-udp', 'passed']]
+    assert results == [
+        DiagnosticCheck('daemon-running-test-unit',
+                        'Service {service_name} is running', Result.PASSED,
+                        {'service_name': 'test-unit'}),
+        DiagnosticCheck('test-result-8273-tcp4', 'test-result-8273-tcp4',
+                        Result.PASSED),
+        DiagnosticCheck('test-result-345-udp', 'test-result-345-udp',
+                        Result.PASSED)
+    ]
     port_listening.assert_has_calls([call(8273, 'tcp4'), call(345, 'udp')])
     service_is_running.assert_has_calls([call('test-unit')])
 
     service_is_running.return_value = False
     results = daemon.diagnose()
-    assert results[0][1] == 'failed'
+    assert results[0].result == Result.FAILED
 
 
 @patch('plinth.action_utils.service_is_running')
@@ -161,15 +215,37 @@ def test_diagnose_port_listening(connections):
 
     # Check that message is correct
     results = diagnose_port_listening(1234)
-    assert results == ['Listening on tcp port 1234', 'passed']
+    assert results == DiagnosticCheck('daemon-listening-tcp-1234',
+                                      'Listening on {kind} port {port}',
+                                      Result.PASSED, {
+                                          'kind': 'tcp',
+                                          'port': 1234
+                                      })
     results = diagnose_port_listening(1234, 'tcp', '0.0.0.0')
-    assert results == ['Listening on tcp port 0.0.0.0:1234', 'passed']
+    assert results == DiagnosticCheck(
+        'daemon-listening-address-tcp-1234-0.0.0.0',
+        'Listening on {kind} port {listen_address}:{port}', Result.PASSED, {
+            'kind': 'tcp',
+            'port': 1234,
+            'listen_address': '0.0.0.0'
+        })
 
     # Failed results
     results = diagnose_port_listening(4321)
-    assert results == ['Listening on tcp port 4321', 'failed']
+    assert results == DiagnosticCheck('daemon-listening-tcp-4321',
+                                      'Listening on {kind} port {port}',
+                                      Result.FAILED, {
+                                          'kind': 'tcp',
+                                          'port': 4321
+                                      })
     results = diagnose_port_listening(4321, 'tcp', '0.0.0.0')
-    assert results == ['Listening on tcp port 0.0.0.0:4321', 'failed']
+    assert results == DiagnosticCheck(
+        'daemon-listening-address-tcp-4321-0.0.0.0',
+        'Listening on {kind} port {listen_address}:{port}', Result.FAILED, {
+            'kind': 'tcp',
+            'port': 4321,
+            'listen_address': '0.0.0.0'
+        })
 
     # Check if psutil call is being made with right argument
     results = diagnose_port_listening(1234, 'tcp')
@@ -186,26 +262,30 @@ def test_diagnose_port_listening(connections):
     connections.assert_called_with('udp6')
 
     # TCP
-    assert diagnose_port_listening(1234)[1] == 'passed'
-    assert diagnose_port_listening(1000)[1] == 'failed'
-    assert diagnose_port_listening(2345)[1] == 'failed'
-    assert diagnose_port_listening(1234, 'tcp', '0.0.0.0')[1] == 'passed'
-    assert diagnose_port_listening(1234, 'tcp', '1.1.1.1')[1] == 'failed'
-    assert diagnose_port_listening(1234, 'tcp6')[1] == 'passed'
-    assert diagnose_port_listening(1234, 'tcp4')[1] == 'passed'
-    assert diagnose_port_listening(6789, 'tcp4')[1] == 'passed'
-    assert diagnose_port_listening(5678, 'tcp4')[1] == 'failed'
+    assert diagnose_port_listening(1234).result == Result.PASSED
+    assert diagnose_port_listening(1000).result == Result.FAILED
+    assert diagnose_port_listening(2345).result == Result.FAILED
+    assert diagnose_port_listening(1234, 'tcp',
+                                   '0.0.0.0').result == Result.PASSED
+    assert diagnose_port_listening(1234, 'tcp',
+                                   '1.1.1.1').result == Result.FAILED
+    assert diagnose_port_listening(1234, 'tcp6').result == Result.PASSED
+    assert diagnose_port_listening(1234, 'tcp4').result == Result.PASSED
+    assert diagnose_port_listening(6789, 'tcp4').result == Result.PASSED
+    assert diagnose_port_listening(5678, 'tcp4').result == Result.FAILED
 
     # UDP
-    assert diagnose_port_listening(3456, 'udp')[1] == 'passed'
-    assert diagnose_port_listening(3000, 'udp')[1] == 'failed'
-    assert diagnose_port_listening(4567, 'udp')[1] == 'failed'
-    assert diagnose_port_listening(3456, 'udp', '0.0.0.0')[1] == 'passed'
-    assert diagnose_port_listening(3456, 'udp', '1.1.1.1')[1] == 'failed'
-    assert diagnose_port_listening(3456, 'udp6')[1] == 'passed'
-    assert diagnose_port_listening(3456, 'udp4')[1] == 'passed'
-    assert diagnose_port_listening(6789, 'udp4')[1] == 'passed'
-    assert diagnose_port_listening(5678, 'udp4')[1] == 'failed'
+    assert diagnose_port_listening(3456, 'udp').result == Result.PASSED
+    assert diagnose_port_listening(3000, 'udp').result == Result.FAILED
+    assert diagnose_port_listening(4567, 'udp').result == Result.FAILED
+    assert diagnose_port_listening(3456, 'udp',
+                                   '0.0.0.0').result == Result.PASSED
+    assert diagnose_port_listening(3456, 'udp',
+                                   '1.1.1.1').result == Result.FAILED
+    assert diagnose_port_listening(3456, 'udp6').result == Result.PASSED
+    assert diagnose_port_listening(3456, 'udp4').result == Result.PASSED
+    assert diagnose_port_listening(6789, 'udp4').result == Result.PASSED
+    assert diagnose_port_listening(5678, 'udp4').result == Result.FAILED
 
 
 @patch('subprocess.Popen')
@@ -213,21 +293,32 @@ def test_diagnose_netcat(popen):
     """Test running diagnostic test using netcat."""
     popen().returncode = 0
     result = diagnose_netcat('test-host', 3300, input='test-input')
-    assert result == ['Connect to test-host:3300', 'passed']
+    parameters = {'host': 'test-host', 'port': 3300, 'negate': False}
+    assert result == DiagnosticCheck('daemon-netcat-test-host-3300',
+                                     'Connect to test-host:3300',
+                                     Result.PASSED, parameters)
     assert popen.mock_calls[1][1] == (['nc', 'test-host', '3300'], )
     assert popen.mock_calls[2] == call().communicate(input=b'test-input')
 
     result = diagnose_netcat('test-host', 3300, input='test-input',
                              negate=True)
-    assert result == ['Cannot connect to test-host:3300', 'failed']
+    parameters2 = parameters.copy()
+    parameters2['negate'] = True
+    assert result == DiagnosticCheck('daemon-netcat-negate-test-host-3300',
+                                     'Cannot connect to test-host:3300',
+                                     Result.FAILED, parameters2)
 
     popen().returncode = 1
     result = diagnose_netcat('test-host', 3300, input='test-input')
-    assert result == ['Connect to test-host:3300', 'failed']
+    assert result == DiagnosticCheck('daemon-netcat-test-host-3300',
+                                     'Connect to test-host:3300',
+                                     Result.FAILED, parameters)
 
     result = diagnose_netcat('test-host', 3300, input='test-input',
                              negate=True)
-    assert result == ['Cannot connect to test-host:3300', 'passed']
+    assert result == DiagnosticCheck('daemon-netcat-negate-test-host-3300',
+                                     'Cannot connect to test-host:3300',
+                                     Result.PASSED, parameters2)
 
 
 def test_related_daemon_initialization():
